// ExactLike.c
// Tim Brady, 2011
//
// Drop in replacement for the ExactLike function in 
// E2/Summary_ChangeDetection.m
//
// Calculates the likelihood of the display
//  curDisplay under the MRF parameters params. params[0] is the horizontal 
//  weight; params[1] is the vertical weight.
//
//

#include "mex.h"
#include <math.h>
#define roundInt(x) ((int)((x)+0.5))

void mexFunction(int nlhs, mxArray *plhs[], int nrhs,
                 const mxArray *prhs[])
{
  int nitems, item1, item2, color;
  double *pParams, *y, *locProb;
  double *pLocations, *pColors;
  int hNeighborSum[8];
  int vNeighborSum[8];
  double curProb;
  double likeVal[8];
  double totalLikeVal;
  double probCur;
  
  /* Check for proper number of arguments. */
  if (nrhs != 3 || nlhs > 2) {
    mexErrMsgTxt("Invalid calling method ... must have 3 right hand "
	 "arguments (params, locations, colors) and no more than 2 left hand arguments");
  } 
  
  /* Parameters they passed in */
  pParams = (double*)mxGetPr(prhs[0]);

  /* Get information about locations and colors */
  nitems = mxGetM(prhs[1]);
  pLocations = (double*)mxGetPr(prhs[1]);
  pColors = (double*)mxGetPr(prhs[2]);
  
  /* Create output matrix and get a pointer to it */
  plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
  y = mxGetPr(plhs[0]);
  plhs[1] = mxCreateDoubleMatrix(nitems, 1, mxREAL);  
  locProb = mxGetPr(plhs[1]);
   
  curProb = 0;
  for (item1=0; item1<nitems; item1++)
  {
      /* item1 is the item under consideration */
      totalLikeVal = 0;
      for (color=1; color<=7; color++) {
          
          /* for each color it could be */
          hNeighborSum[color] = 0;
          vNeighborSum[color] = 0;
          likeVal[color] = 0;
          
          for (item2=0; item2<nitems; item2++)
          {             
              /* is horiz-neighbor? */
              if (roundInt(pLocations[item2])==roundInt(pLocations[item1])+4) {
                  if (roundInt(pColors[item2]) == color) {
                      hNeighborSum[color]++;
                  }
              }              
              /* is vert-neighbor? */
              if (roundInt(pLocations[item2])==roundInt(pLocations[item1])+1 && 
                      ceil(pLocations[item2]/4)==ceil(pLocations[item1]/4)) {
                  if (roundInt(pColors[item2]) == color) {
                      vNeighborSum[color]++;
                  }
              }                
              
              /* is horiz-neighbor? */
              if (roundInt(pLocations[item1])==roundInt(pLocations[item2])+4) {
                  if (roundInt(pColors[item2]) == color) {
                      hNeighborSum[color]++;
                  }
              }              
              /* is vert-neighbor? */
              if (roundInt(pLocations[item1])==roundInt(pLocations[item2])+1 && 
                      ceil(pLocations[item1]/4)==ceil(pLocations[item2]/4)) {
                  if (roundInt(pColors[item2]) == color) {
                      vNeighborSum[color]++;
                  }
              }   
              
          }           
          likeVal[color] = exp(pParams[0]*hNeighborSum[color] + pParams[1]*vNeighborSum[color]);
          totalLikeVal += likeVal[color];
      }
      
      /* Compute the overall probability of the current value and the
       * other value */
      probCur = likeVal[roundInt(pColors[item1])];
              
      /* Sum the log probabilities of each location to get the total
       * probability of a given display */
      locProb[item1] = probCur/totalLikeVal;
      curProb += log(locProb[item1]);
  }
  
  /* Return the total probability of the display */
  y[0] = exp(curProb);
}



